#!/usr/bin/ruby -w
# -*- ruby -*-

class GlarkHelp

  def initialize
    puts "USAGE"
    puts "    glark [options] expression file..."
    puts ""
    puts "OPTIONS"
    puts "    -?, --help"
    puts "        Display this help message."
    puts ""
    puts "    -A NUM, --after-context=NUM"
    puts "        Print NUM lines after a matched expression."
    puts ""
    puts "    -b NUM[%], --before NUM[%]"
    puts "        Restrict the search to before the given location, which represents"
    puts "        either the number of the last line within the valid range, or the"
    puts "        percentage of lines to be searched."
    puts ""
    puts "    --binary-files=TYPE"
    puts "        Specify how to treat binary files, thus overriding the default"
    puts "        behavior, which is to denote the binary files that match the"
    puts "        expression, without displaying the match."
    puts ""
    puts "    -B NUM, --before-context=NUM"
    puts "        Print NUM lines before a matched expression."
    puts ""
    puts "    -C [NUM], -NUM, --context[=NUM]"
    puts "        Output <NUM> lines of context around a matched expression. The"
    puts "        default is no context. If no NUM is given for this option, the"
    puts "        number of lines of context is 2."
    puts ""
    puts "    -c, --count"
    puts "        Instead of normal output, display only the number of matches in"
    puts "        each file. Because of multi-line matches, the -v (--invert-match)"
    puts "        option would produce misleading output with this option."
    puts ""
    puts "    -d ACTION, --directories=ACTION"
    puts "        Directories are processed according to the given ACTION, which by"
    puts "        default is 'read'. If ACTION is 'recurse', each file in the"
    puts "        directory is read and each subdirectory is recursed into"
    puts "        (equivalent to the -r option). If ACTION is 'skip', directories are"
    puts "        not read, and no message is produced."
    puts ""
    puts "    -f NUM[%], --after NUM[%]"
    puts "        Restrict the search to after the given location, which represents"
    puts "        either the number of the first line within the valid range, or the"
    puts "        percentage of lines to be skipped."
    puts ""
    puts "    -F, --file-color COLOR"
    puts "        Specify the highlight color for file names."
    puts ""
    puts "    -g, --grep"
    puts "        Produce output like the grep default: file names and line numbers,"
    puts "        and a single line of the match, which will be the first line for."
    puts "        matches that span multiple lines."
    puts ""
    puts "    -h, --no-filename"
    puts "        Do not display the names of the files that matched."
    puts ""
    puts "    -H, --with-filename"
    puts "        Display the names of the files that matched. This is the default"
    puts "        behavior."
    puts ""
    puts "    -i, --ignore-case"
    puts "        Match regular expressions without regard to case. The default is"
    puts "        case sensitive."
    puts ""
    puts "    -l, --files-with-matches"
    puts "        Print only the names of the file that matched the expression."
    puts ""
    puts "    -L, --files-without-match"
    puts "        Print only the names of the file that did not match the expression."
    puts ""
    puts "    -m NUM, --match-limit NUM"
    puts "        Find only the first NUM matches in each file."
    puts ""
    puts "    -M, --exclude-matching-filenames"
    puts "        Do not search files whose names match the given expression."
    puts ""
    puts "    -n, --line-number"
    puts "        Display the line numbers. This is the default behavior."
    puts ""
    puts "    -N, --no-line-number"
    puts "        Do not display the line numbers."
    puts ""
    puts "    -q, -s, --quiet, --no-messages"
    puts "        Suppress warnings."
    puts ""
    puts "    -Q, --no-quiet"
    puts "        Enable warnings. This is the default."
    puts ""
    puts "    -r, --recurse"
    puts "        Recurse through directories. Equivalent to --directories=read."
    puts ""
    puts "    -R, --range NUM[%] NUM[%]"
    puts "        Restrict the search to the given range of lines."
    puts ""
    puts "    -T, --text-color COLOR"
    puts "        Specify the highlight color for text."
    puts ""
    puts "    -u, --highlight, -hl"
    puts "        Turn highlighting on, which uses ANSI escape sequences. This is the"
    puts "        default behavior. The '-hl' option is deprecated."
    puts ""
    puts "    -U, --no-highlight, -nohl"
    puts "        Turn highlighting off. The '-nohl' option is deprecated."
    puts ""
    puts "    -v, --invert-match"
    puts "        Show lines that do not match the expression."
    puts ""
    puts "    -V, --version"
    puts "        Display version information."
    puts ""
    puts "    -w, --word, --word-regexp"
    puts "        Put word boundaries around each pattern, thus matching only where"
    puts "        the full word(s) occur in the text. Thus, glark -w Foo is the same"
    puts "        as glark '/\bFoo\b/'."
    puts ""
    puts "    -W, --no-word"
    puts "        Do not put word boundaries around each pattern. This is the default."
    puts ""
    puts "    -x, --line-regexp"
    puts "        Select only where the entire line matches the pattern(s)."
    puts ""
    puts "    -X, --no-line-regexp"
    puts "        Do not select only where the entire line matches the pattern(s). This"
    puts "        is the default behavior."
    puts ""
    puts "    -Z, --null"
    puts "        When in -l mode, write file names followed by the ASCII NUL character ('\\0')"
    puts "        instead of '\\n'."
    puts ""
    puts "    -_, --verbose"
    puts "        Display normally suppressed output, for debugging purposes."
    puts ""
    puts "EXPRESSION"
    puts "    A regular expression, in Ruby format. Run `perldoc perlre' for more"
    puts "    general information. The expression may be of either form:"
    puts ""
    puts "        something"
    puts "        /something/"
    puts ""
    puts "    All regular expression characters and options are available, such as"
    puts "    ``\w'', ``.*?'' and ``[^9]''."
    puts ""
    puts "    If the and and or options are not used, the last non-option is"
    puts "    considered to be the expression to be matched."
    puts ""
    puts "    -o expr1 expr2"
    puts "    --or expr1 expr2 --end-of-or"
    puts "        Matches either of the two expressions. The results of the two forms"
    puts "        are equivalent. In the latter syntax, the --end-of-or is optional."
    puts ""
    puts "    -a number expr1 expr2"
    puts "    --and number expr1 expr2 --end-of-and"
    puts "        Matches both of the two expressions, within <number> lines of each"
    puts "        other. As with the or option, the results of the two forms are"
    puts "        equivalent, and the --end-of-and is optional."
    puts ""
    puts "        If the number provided is -1 (negative one), the distance is"
    puts "        considered to be ``infinite'', and thus, the condition is satisfied"
    puts "        if both expressions match within the same file."
    puts ""
    puts "        If the number provided is 0 (zero), the condition is satisfied if"
    puts "        both expressions match on the same line."
    puts ""
    puts "        A warning will be issued if the value given in the number position"
    puts "        does not appear to be numeric."
    puts ""
    puts "    --explain"
    puts "        Writes the regular expression in a more legible format. Useful for"
    puts "        debugging."
    puts ""
    puts "See the man page for more information."
    puts ""
  end

end


if __FILE__ == $0
  GlarkHelp.new
end
