#!/bin/sh
exec ruby -w -x $0 ${1+"$@"} # -*- ruby -*-
#!ruby -w

# An extended grep, with extended functionality including full regular
# expressions, contextual output, highlighting, detection and exclusion of
# nontext files, and complex matching criteria.

# $Id: glarkrun.rb,v 1.4 2003/05/05 13:43:54 jeugenepace Exp $

require "English"

$stdout.sync = true             # unbuffer
$stderr.sync = true             # unbuffer

$PACKAGE = "glark"
$VERSION = "1.6.4"

require './glarkfile'
require './filetester'

# The main processor.
class Glark 
  def initialize(func)
    @func = func
  end

  def search_file(fname, lines)
    Log.log "searching #{fname} for #{@func}" if $options.verbose
    p = InputFile.new(fname, lines)
    @func.process(p)
  end

  def search_binary_file(fname, lines)
    Log.log "searching binary file #{fname} for #{@func}" if $options.verbose
    bf = BinaryFile.new(fname, lines)
    @func.process(bf)
  end
  
  def search(name)
    Log.log "searching #{name} for #{@func}"

    if name == "-" 
      Log.log "reading standard input..."
      $stderr.print "reading standard input...\n" unless $options.quiet
      search_file(name, $stdin.readlines)
    else
      type = FileTester.type(name)
      case type
      when FileTester::TEXT
        Log.log "searching text", 2
        search_file(name, IO.readlines(name))
      when FileTester::BINARY
        Log.log "handling binary", 1
        # Log.print "not a text file: #{name}\n"
        
        case $options.binary_files
        when "without-match"
          Log.log "skipping binary file #{name}"
          
        when "binary"
          search_binary_file(name, IO.readlines(name))
          
        when "text"
          Log.log "processing binary file #{name} as text"
          search_file(name, IO.readlines(name))
        end
        
      when FileTester::UNREADABLE
        Log.log "skipping unreadable", 2
        $stderr.print "file not readable: #{name}\n" unless $options.quiet
      when FileTester::NONE
        Log.log "skipping none", 2
        $stderr.print "WARNING: no such file: #{name}\n" unless $options.quiet
      when FileTester::UNKNOWN
        Log.log "skipping unknown", 2
        $stderr.print "WARNING: unknown file type: #{name}\n" unless $options.quiet
      when FileTester::DIRECTORY
        Log.log "processing directory"
        case $options.directory
        when "read"
          $stderr.print "glark: #{name}: Is a directory\n" unless $options.quiet
        when "recurse"
          entries = Dir.entries(name).reject { |x| x == "." || x == ".." }
          entries.each do |e|
            search(name + "/" + e)
          end
        when "skip"
          Log.print "skipping directory #{name}"
        end
      else
        print "unknown type #{type}"
      end
    end
  end
end

require './glarkopt'

begin
  Log.set_widths(15, 5, -30)

  Log.log "loading options"
  GlarkOptions.new($PACKAGE, $VERSION).run
  Log.log "done loading options"

  # To get rid of the annoying stack trace on ctrl-C:
  trap("INT") { abort }

  puts $options.expr if $options.explain

  glark = Glark.new($options.expr)
  files = if ARGV.size > 0 then ARGV else [ '-' ] end

  files.each do |f|
    if $options.exclude_matching_files
      if $options.expr.evaluate(f, 0, 0)
        Log.log "skipping file #{f} with matching name"
        next
      else
        Log.log "not skipping file #{f}"
      end
    end
    glark.search(f) 
  end
rescue => e
  # show only the message, not the stack trace:
  $stderr.puts "error: #{e}"
end
